import http from 'http';

interface CompatibilityRequestProps {
    event: H3Event;
    context: H3EventContext;
    /** Only available with connect and press */
    originalUrl?: string;
}
interface IncomingMessage extends http.IncomingMessage, CompatibilityRequestProps {
    req: H3Event['req'];
    res: H3Event['res'];
}
interface ServerResponse extends http.ServerResponse {
    event: H3Event;
    res: H3Event['res'];
    req: http.ServerResponse['req'] & CompatibilityRequestProps;
}
declare type Handler<T = any, ReqT = {}> = (req: IncomingMessage & ReqT, res: ServerResponse) => T;
declare type Middleware = (req: IncomingMessage, res: ServerResponse, next: (err?: Error) => any) => any;
interface H3EventContext extends Record<string, any> {
}
declare type HandlerResponse<T = any> = T | Promise<T>;
interface EventHandler<T = any> {
    '__is_handler__'?: true;
    (event: H3Event): HandlerResponse<T>;
}
declare type LazyEventHandler = () => EventHandler | Promise<EventHandler>;
declare type CompatibilityEventHandler = EventHandler | Handler | Middleware;

declare class H3Headers implements Headers {
    _headers: Record<string, string>;
    constructor(init?: HeadersInit);
    append(name: string, value: string): void;
    delete(name: string): void;
    get(name: string): string | null;
    has(name: string): boolean;
    set(name: string, value: string): void;
    forEach(callbackfn: (value: string, key: string, parent: Headers) => void): void;
}

declare class H3Response implements Response {
    readonly headers: H3Headers;
    readonly status: number;
    readonly statusText: string;
    readonly redirected: boolean;
    readonly ok: boolean;
    readonly url: string;
    _body: string | ArrayBuffer | Uint8Array;
    readonly body: ReadableStream<Uint8Array> | null;
    readonly type: ResponseType;
    readonly bodyUsed = false;
    constructor(body?: BodyInit | HandlerResponse | null, init?: ResponseInit);
    clone(): H3Response;
    arrayBuffer(): Promise<ArrayBuffer>;
    blob(): Promise<Blob>;
    formData(): Promise<FormData>;
    json<T = any>(): Promise<T>;
    text(): Promise<string>;
}

declare class H3Event implements Pick<FetchEvent, 'respondWith'> {
    '__is_event__': boolean;
    req: IncomingMessage;
    res: ServerResponse;
    event: H3Event;
    context: H3EventContext;
    constructor(req: http.IncomingMessage | IncomingMessage, res: http.ServerResponse | ServerResponse);
    respondWith(r: H3Response | PromiseLike<H3Response>): void;
}
declare function isEvent(input: any): input is H3Event;
declare function createEvent(req: http.IncomingMessage, res: http.ServerResponse): H3Event;

declare function defineEventHandler<T = any>(handler: EventHandler<T>): EventHandler<T>;
declare const eventHandler: typeof defineEventHandler;
declare function isEventHandler(input: any): input is EventHandler;
declare function toEventHandler(handler: CompatibilityEventHandler): EventHandler;
interface DynamicEventHandler extends EventHandler {
    set: (handler: EventHandler) => void;
}
declare function dynamicEventHandler(initial?: EventHandler): DynamicEventHandler;
declare function defineLazyEventHandler(factory: LazyEventHandler): EventHandler;
declare const lazyEventHandler: typeof defineLazyEventHandler;

export { DynamicEventHandler, H3Event, H3Headers, H3Response, createEvent, defineEventHandler, defineLazyEventHandler, dynamicEventHandler, eventHandler, isEvent, isEventHandler, lazyEventHandler, toEventHandler };
